<?php

/**
 * @file
 * Helper functions.
 */
use Drupal\mcapi\Entity\WalletInterface;
use Drupal\mcapi\Entity\Currency;
use Drupal\mcapi\Entity\CurrencyInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;

/**
 * Preprocessor function for theme callback mcapi_limits_absolute.
 */
function template_preprocess_mcapi_limits_absolute(&$vars) {
  $currency = $vars['currency'];
  $curr_id = $currency->id();
  $summary = $vars['wallet']->getStats($currency->id());
  $limits = mcapi_limits_wallet_limiter($vars['wallet']);
  $vars['balanceformatted'] = ['#markup' => $currency->format($summary->balance)];
  $min = $limits->min($curr_id);
  $max = $limits->max($curr_id);
  if (is_null($min)) {
    $vars['minformatted'] = [];
    $vars['min'] = NULL;
  }
  else {
    $vars['minformatted'] = ['#markup' => $currency->format($min)];
    $vars['min'] = $currency->format($limits->min($curr_id), CurrencyInterface::DISPLAY_PLAIN);
  }
  if (is_null($max)) {
    $vars['maxformatted'] = [];
    $vars['max'] = NULL;
  }
  else {
    $vars['maxformatted'] = ['#markup' => $currency->format($max)];
    $vars['max'] = $currency->format($limits->max($curr_id), CurrencyInterface::DISPLAY_PLAIN);
  }
  $vars['balance'] = $currency->format($summary->balance, CurrencyInterface::DISPLAY_PLAIN);
}

/**
 * Preprocessor function for theme callback mcapi_limits_relative.
 */
function template_preprocess_mcapi_limits_relative(&$vars) {
  $balance = $vars['wallet']->getStat($vars['currency']->id(), 'balance');
  $limiter = mcapi_limits_wallet_limiter($vars['wallet']);

  $vars['earn_limit'] = $limiter->earnLimit($vars['currency']->id(), $balance);
  $vars['spend_limit'] = $limiter->spendLimit($vars['currency']->id(), $balance);

  if (!is_null($vars['spend_limit'])) {
    $vars['spendlimitformatted'] = ['#markup' => $vars['currency']->format($vars['spend_limit'])];
  }
  if (!is_null($vars['earn_limit'])) {
    $vars['earnlimitformatted'] = ['#markup' => $vars['currency']->format($vars['earn_limit'])];
  }
}

/**
 * Preprocessor function for theme callback mcapi_limits_balanced.
 *
 * By design this can only be used if the max and min are equidistant from zero
 * Also because of limitations of the google charts gauge visualisation,
 * this can only be used if the formatted value is numeric.
 */
function template_preprocess_mcapi_limits_balanced(&$vars) {
  $currency = $vars['currency'];
  // The google gauge cannot mask the real number with a formatted value.
  $limiter = mcapi_limits_wallet_limiter($vars['wallet']);
  $vars['balance'] = $currency->format(
    $vars['wallet']->getStats($currency->id())->balance,
    CurrencyInterface::DISPLAY_PLAIN
  );
  $min = $limiter->min($currency->id());
  $max = $limiter->max($currency->id());
  $vars['min'] = $currency->format($min, CurrencyInterface::DISPLAY_PLAIN);
  $vars['max'] = $currency->format($max, CurrencyInterface::DISPLAY_PLAIN);
  $vars['minformatted'] = $currency->format($min);
  $vars['maxformatted'] = $currency->format($max);
}

/**
 * Implements hook_form_MCAPI_CURRENCY_EDIT_FORM_alter().
 *
 * Add the limits settings to the currency form.
 */
function _mcapi_limits_form_mcapi_currency_edit_form_alter(&$form, FormStateInterface $form_state) {
  $form['limits'] = [
    '#title' => t('Balance limits'),
    '#description' => t("Transactions will be rejected which take accounts beyond these limits."),
    '#type' => 'details',
    '#weight' => 20,
    '#collapsible' => FALSE,
    '#open' => TRUE,
  ];
  $plugin = mcapi_limits_form_state_load_plugin(
    $form_state->getFormObject()->getEntity(),
    $form_state->getValue('plugin')
  );

  $form['limits'] += [
    'plugin' => [
      '#title' => t('Plugin'),
      '#description' => t('How will the limits be determined?'),
      '#type' => 'select',
      '#options' => \Drupal::service('plugin.manager.mcapi_limits')->optionsList(),
      // 'none' is the default plugin name.
      '#default_value' => $plugin->id,
      '#weight' => -1,
      '#ajax' => [
        'callback' => 'mcapi_limits_plugin_settings_subform',
        'wrapper' => 'limits-plugin-settings',
        'effect' => 'fade',
      ],
    ],
  ];
  // Show the options form for the selected plugin.
  $form['limits']['plugin_settings'] = $plugin->commonConfigurationForm([], $form_state) + [
    $plugin->id => $plugin->buildConfigurationForm([], $form_state),
    '#prefix' => '<div id="limits-plugin-settings">',
    '#suffix' => '</div>',
    '#tree' => TRUE
  ];
  // This is the ajax wrapper.
  $form['actions']['submit']['#validate'][] = 'mcapi_limits_validate';
  $form['actions']['submit']['#submit'][] = 'mcapi_limits_submit';
}

/**
 * Get the limits plugin.
 *
 * @param CurrencyInterface $currency
 *   The currency.
 * @param string $plugin_id
 *
 * @return \Drupal\mcapi_limits\Plugin\McapiLimitsInterface
 *   The Limits object.
 */
function mcapi_limits_form_state_load_plugin(CurrencyInterface $currency, string $plugin_id = NULL) {
  $limitsManager = \Drupal::service('plugin.manager.mcapi_limits');
  if ($plugin_id) {
    $plugin = $limitsManager->createInstance($plugin_id, ['currency' => $currency]);
  }
  else {
    $plugin = $limitsManager->createInstanceCurrency($currency);
  }
  return $plugin;
}

/**
 * Ajax callback.
 */
function mcapi_limits_validate(&$form, FormStateInterface $form_state) {
  $currency = $form_state->getFormObject()->getEntity();
  mcapi_limits_form_state_load_plugin($currency, $form_state->getValue('plugin'))
    ->validateConfigurationForm($form, $form_state);
}

/**
 * Form submission callback on the submit button.
 *
 * Save the plugin settings as config.
 */
function mcapi_limits_submit(&$form, FormStateInterface $form_state) {
  $currency = $form_state->getFormObject()->getEntity();
  $plugin = mcapi_limits_form_state_load_plugin($currency, $form_state->getValue('plugin'));
  $plugin->submitConfigurationForm($form, $form_state);

  // Rendered wallets may be affected by changes in balance limits.
  Cache::invalidateTags(['mcapi_wallet_view']);

  $config = $plugin->getConfiguration();
  $plugin_config = $config[$plugin->id]??[];
  unset($config[$plugin->id]);
  foreach ($config as $key => $value) {
    $currency->setThirdPartySetting('mcapi_limits', $key, $value);
  }
  $currency->setThirdPartySetting('mcapi_limits', $plugin->id, $plugin_config);
  // This has to go after the plugin has set the values.
  $currency->setThirdPartySetting('mcapi_limits', 'plugin', $plugin->id);
  $currency->save();
}

/**
 * Ajax callback.
 */
function mcapi_limits_plugin_settings_subform($form, $form_state) {
  return $form['limits']['plugin_settings'];
}

/**
 * Show a wallets limits.
 *
 * Used in hook_wallet_view, views, & blocks.
 *
 * @param WalletInterface $wallet
 *   The wallet.
 * @param array $curr_ids
 *   Limit the display to the currencies given.
 * @param bool $relative
 *   Show the limits relative to the user's current balance.
 *
 * @return array
 *   A renderable array.
 *
 * @todo add the $relative parameter to calling functions, and related config.
 */
function mcapi_view_limits(WalletInterface $wallet, $curr_ids = [], $relative = FALSE) {
  $currencies = Currency::loadMultiple();
  if ($curr_ids) {
    $currencies = array_intersect_key($currencies, array_flip($curr_ids));
  }
  $renderable = [];
  foreach ($currencies as $currency) {
    if ($currency->getThirdPartySetting('mcapi_limits', 'plugin', 'none') == 'none') {
      continue;
    }
    if ($relative) {
      $theme = 'mcapi_limits_relative';
    }
    else {
      $limiter = mcapi_limits_wallet_limiter($wallet);
      $max = $limiter->max($currency->id());
      if ($max && (-$max == $limiter->min($currency->id()))) {
        $theme = 'mcapi_limits_balanced';
      }
      else {
        $theme = 'mcapi_limits_absolute';
      }
    }
    $renderable[$currency->id()] = [
      '#theme' => $theme,
      '#wallet' => $wallet,
      '#currency' => $currency,
    ];
  }
  return $renderable;
}
